#include "chpstick_constants.h"
#include <Servo.h>
#include <chpstick_4BeatWalking>
#include <chpstick_ServoMath>
#include <logic3_nunchuck>
#include "BlinkM_funcs.h"
#include <Wire.h>

// necessary servo objects
Servo HFR; // name convention:
Servo KFR; // char 1 2 3
Servo HFL; //      | | |
Servo KFL; //      | | -- R/L = right / left
Servo HRL; //      | ---- F/R = front / rear
Servo KRL; //      ------ H/K = hip   / knee
Servo HRR; //
Servo KRR; //
Servo EYE; // yeah right... EYE servo does not follow the convention ;)

// BlinkM stuff
int blinkM = 0x09; //I2C address
int blinkMscript = 5; // script to start.. wont start again if already started
// some predefined colours
#define indicateWhite BlinkM_setRGB(blinkM, 0x44, 0x44, 0x44)
#define indicateBlack BlinkM_setRGB(blinkM, 0x00, 0x00, 0x00)
#define indicateRed   BlinkM_setRGB(blinkM, 0xFF, 0x00, 0x00)
#define indicateGreen BlinkM_setRGB(blinkM, 0x00, 0xFF, 0x00)

// button pin on the mini shield
int pinBtnInit = 11;
boolean initDone = false;
boolean runAutomomous = false;

// Timer for autonomous control
static unsigned long msTick = 0;
int timeOutSec = 30; // half of it and we enter bored mode

//  IR distance sensor
int EYEsensor = A0;  // analog pin for input

// delays for walking pattern
int stepDelay = 0; // the delay between each leg's servo move
int moveDelay = 1; // the wait time in the for loop while the servo move is ongoing (dont put it to 0, use moveServoFAST instead)
int stepDelayMin = 0;
int stepDelayMax = 1000;
int moveDelayMin = 1;
int moveDelayMax = 15;

void setup(){
	
	Serial.begin(115200);
	
	msTick = millis(); // init millisecond counter
	
	// nunchuck stuff
	// init button goes to 0 if active
	pinMode(pinBtnInit, INPUT_PULLUP);
	// fire up I2C interface
	nunchuckSetPowerpins(); // use analog pins 2&3 as fake gnd & pwr
	
	// BlinkM stuff
	BlinkM_beginWithPower();
	BlinkM_setStartupParams(blinkM, 1, scriptBlue, 0, 5, -10);
	
	// servo stuff
	//hips
	HFR.attach(idHFR); // see constants.h for id reference
	HFL.attach(idHFL);
	HRL.attach(idHRL);
	HRR.attach(idHRR);
  
	//legs
	KFR.attach(idKFR);
	KFL.attach(idKFL);
	KRL.attach(idKRL);
	KRR.attach(idKRR);
  
	// head
	EYE.attach(idEYE); 
	pinMode(EYEsensor, INPUT);

	//start positions
	moveServo(HFR, idHFR, FhipFWD, moveDelay);
	moveServo(KFR, idKFR, FTG, moveDelay);
	moveServo(HFL, idHFL, FhipFWD, moveDelay);
	moveServo(KFL, idKFL, FTG, moveDelay);
	moveServo(HRL, idHRL, RhipREV, moveDelay);
	moveServo(KRL, idKRL, FTG, moveDelay);
	moveServo(HRR, idHRR, RhipREV, moveDelay);
	moveServo(KRR, idKRR, FTG, moveDelay);
	moveServo(EYE, idEYE, EYE_center, moveDelay);
}

void loop(){
	
	// check if init button is pressed 
	// if true perform nunchuck init
	if (!digitalRead(pinBtnInit) == true && !initDone){
		nunchuckInit();
		if (isNunchuckInitialized()){ // successfully initialized
			initDone = true;
			runAutomomous = false;
			
			for (int i = 0; i < 7; i++){
				indicateWhite; delay(80);
				indicateBlack; delay(80);
			}
			DoBlinkMscript(scriptGreen, 10, 0);
			nunchuckUpdate(); // first update is garbage
			msTick = millis();
		}else{ // no nunchuck present run autonomous
			DoBlinkMscript(scriptMagenta, 10, 0);
			runAutomomous = true;
		}// nunchuck initialized or not
	}// init Btn pressed
	
	if (initDone){
		nunchuckUpdate();
		//Serial.print("aX: ");Serial.print(readAccelX());Serial.print("\t");
		//Serial.print("aY: ");Serial.print(readAccelY());Serial.print("\t");
		//Serial.print("aZ: ");Serial.print(readAccelZ());Serial.print("\t");
		//Serial.print("jX: ");Serial.print(readJoyX());Serial.print("\t");
		//Serial.print("jY: ");Serial.print(readJoyY());Serial.print("\t");
		//Serial.print("cBtn: ");Serial.print(cButtonPressed());Serial.print("\t");
		//Serial.print("zBtn: ");Serial.print(zButtonPressed());Serial.print("\t");
		//Serial.print("jL: ");Serial.print(JoyLeft());Serial.print("\t");
		//Serial.print("jR: ");Serial.print(JoyRight());Serial.print("\t");
		//Serial.print("jU: ");Serial.print(JoyUp());Serial.print("\t");
		//Serial.print("jD: ");Serial.print(JoyDown());Serial.println("\t");
		
		//Serial.println(millis() - msTick);
		
		if (zButtonPressed() && cButtonPressed()){
			if (JoyLeft()) decreaseMoveDelay(), msTick = millis();
			if (JoyRight()) increaseMoveDelay(), msTick = millis();
			if (JoyUp()) increaseStepDelay(), msTick = millis();
			if (JoyDown()) decreaseStepDelay(), msTick = millis();
			return;
		}// z+c Btn
		
		if (zButtonPressed()){
			indicateWhite, msTick = millis();
			if (obstacleFound(threshold) && !runAutomomous) escape(), msTick = millis();
			if (JoyLeft()) crabLeft4Beat(stepDelay, moveDelay), msTick = millis();
			if (JoyRight())	crabRight4Beat(stepDelay, moveDelay), msTick = millis();
			return;
		}else{
			if (obstacleFound(threshold) && !runAutomomous){escape(); msTick = millis(); return;}
			if (JoyLeft()){turnLeft4Beat(stepDelay, moveDelay); msTick = millis(); return;}
			if (JoyRight()){turnRight4Beat(stepDelay, moveDelay); msTick = millis(); return;}
		}// zBtn
		
		if (cButtonPressed()){
			indicateWhite, msTick = millis();
			if (JoyUp()) bodyUp(), msTick = millis();
			if (JoyDown()) bodyDown(), msTick = millis();
			return;
		}else{
			if (obstacleFound(threshold) && !runAutomomous){escape(); msTick = millis(); return;}
			if (JoyUp()){forward4Beat(stepDelay, moveDelay); msTick = millis(); return;}
			if (JoyDown()){reverse4Beat(stepDelay, moveDelay); msTick = millis(); return;}	
		}// cBtn
	}// if initDone
	
	if (millis() - msTick > timeOutSec * 1000){
		DoBlinkMscript(scriptMagenta, 10, 0);
		if (!runAutomomous) winkRight(4, moveDelay);
		runAutomomous = true;
	}else if (millis() - msTick > timeOutSec * 500){
		DoBlinkMscript(scriptNeon, 10, 0);
		moveServo(EYE, idEYE, EYE_min + random(EYE_min, EYE_center), random(0, 10));
		delay(random(100, 400));
		moveServo(EYE, idEYE, EYE_max - random(0, EYE_max - EYE_center), random(0, 10));
		delay(random(100, 400));
		moveServo(EYE, idEYE, EYE_center - EYE_center / random(5, 60), random(0, 10));
		delay(random(100, 400));
		moveServo(EYE, idEYE, EYE_center + EYE_center / random(5, 60), random(0, 10));
		delay(random(100, 400));
		moveServo(EYE, idEYE, EYE_center, 1);
	}else{
		runAutomomous = false;
		DoBlinkMscript(scriptGreen, 10, 0);
	}// msTick timer
	
	if (runAutomomous){ 		
		DoBlinkMscript(scriptMagenta, 10, 0);
		
		if (obstacleFound(threshold)){
			escape(); // something in the way
		}else{
			forward4Beat(stepDelay, moveDelay); // nothing in front of the sensor ... so keep beating forward
		}// obstacle found
	}// runAutonomous
}// loop

// map value to colour ramp green->red
void colorIndicator(int val, int valMin, int valMax){
	int red = map(val, valMin, valMax, 0, 255);
	int green = map(val, valMin, valMax, 255, 0);
	
	if (val == valMax){
		indicateRed; 
	}else if (val == valMin){
		indicateGreen; 
	}else{
		BlinkM_setRGB(blinkM, red, green, 0x00);
	}
	delay(80);
}

void increaseStepDelay(){
	stepDelay = constrain(stepDelay + 50, stepDelayMin, stepDelayMax);
	Serial.print("stepDelay: ");
	Serial.println(stepDelay);
	
	colorIndicator(stepDelay, stepDelayMin, stepDelayMax);
}
void decreaseStepDelay(){
	stepDelay = constrain(stepDelay - 50, stepDelayMin, stepDelayMax);
	Serial.print("stepDelay: ");
	Serial.println(stepDelay);
	
	colorIndicator(stepDelay, stepDelayMin, stepDelayMax);
}

void increaseMoveDelay(){
	moveDelay = constrain(moveDelay + 1, moveDelayMin, moveDelayMax);
	Serial.print("moveDelay: ");
	Serial.println(moveDelay);
	
	colorIndicator(moveDelay, moveDelayMin, moveDelayMax);
}
void decreaseMoveDelay(){
	moveDelay = constrain(moveDelay - 1, moveDelayMin, moveDelayMax);
	Serial.print("moveDelay: ");
	Serial.println(moveDelay);
	
	colorIndicator(moveDelay, moveDelayMin, moveDelayMax);
}

void bodyUp(){
	FTG = constrain(FTG + 1, FLG, 90);
	
	colorIndicator(FTG, FLG, 90);
	
	moveServo(KRR, idKRR, FTG, moveDelay);
	moveServo(KRL, idKRL, FTG, moveDelay);
	moveServo(KFR, idKFR, FTG, moveDelay);
	moveServo(KFL, idKFL, FTG, moveDelay);
	Serial.print("FTG: ");
	Serial.println(FTG);
}
void bodyDown(){
	FTG = constrain(FTG - 1, FLG, 90);
	
	colorIndicator(FTG, FLG, 90);
		
	moveServo(KRR, idKRR, FTG, moveDelay);
	moveServo(KRL, idKRL, FTG, moveDelay);
	moveServo(KFR, idKFR, FTG, moveDelay);
	moveServo(KFL, idKFL, FTG, moveDelay);
	Serial.print("FTG: ");
	Serial.println(FTG);
}

void DoBlinkMscript(int scriptID, int fade, int timeAdj){
	
	if (blinkMscript == scriptID) return; // script already running
	
	BlinkM_playScript(blinkM, scriptID, 0, 0);
	BlinkM_setFadeSpeed(blinkM, fade);
	BlinkM_setTimeAdj(blinkM, timeAdj);
	blinkMscript = scriptID;
}

void escape(){
	
	Serial.println("escape");
	
	DoBlinkMscript(scriptRed, 10, 0); // pulse red
	
	int turnTimes = random(1, 5); // randomly turn between 1 and 4 times
	int direction = 0; // -1 = left, 1 = right
	
	// oh sh** something in the way
	direction = obstacleScan();		// lets scan environment
	reverse4Beat(stepDelay, moveDelay);  // crawl back 2 steps to escape
	reverse4Beat(stepDelay, moveDelay);
	
	if (direction == 1){
		switch (random(1, 3)){
			case 1:
				for (int i = 0; i < turnTimes; i++){
					turnRight4Beat(stepDelay, moveDelay);
				}
			break;
			case 2:
				for (int i = 0; i < turnTimes; i++){
					crabRight4Beat(stepDelay, moveDelay);
				}
			break;
		}// switch
	}else if (direction == -1){
		switch (random(1, 3)){
			case 1:
				for (int i = 0; i < turnTimes; i++){
					turnLeft4Beat(stepDelay, moveDelay);
				}
			break;
			case 2:
				for (int i = 0; i < turnTimes; i++){
					crabLeft4Beat(stepDelay, moveDelay);
				}
			break;
		}// switch
	}// choose direction
}// escape

boolean obstacleFound(int thold){
	int distance = 0; // reset distance for each call
	
	// average 5 measurements
	for (int i = 0; i < 5; i++){
		distance += analogRead(EYEsensor);
		delay(2);
	}
	distance = distance / 5;
	
	// if distance gets greater than the threshold we are to close
	if (distance > thold){
		Serial.println("Obstacle found");
		return true;
	}else{
		return false;
	}
}// obstacle found
int obstacleScan(){
	
	// return  1 for left
	// return -1 for right
	
	Serial.println("Scanning...");
	int distanceLeft = 0;
	int distanceRight = 0;
	
	// check left
	moveServo(EYE, idEYE, EYE_min, 2);
	for (int i = 0; i < 5; i++){
		distanceLeft += analogRead(EYEsensor);
		delay(2);
	}
	distanceLeft = distanceLeft / 5;
	delay(100);
	
	// check right
	moveServo(EYE, idEYE, EYE_max, 7);
	for (int i = 0; i < 5; i++){
		distanceRight += analogRead(EYEsensor);
		delay(2);
	}
	distanceRight = distanceRight / 5;
	delay(100);
	
	moveServo(EYE, idEYE, EYE_center, 2);
	
	return (distanceRight > distanceLeft) ? -1 : 1; //voodoo :)
}// obstacle scan
