/*
 * NunchuckPrint
 *
 * 2007 Tod E. Kurt, http://todbot.com/blog/
 *
 * The Wii Nunchuck reading code is taken from Windmeadow Labs
 *   http://www.windmeadow.com/node/42
 *
 *	munkey might have done some adjustments to 
 *  make it work for the chpstck robot
 */
 
#include <Wire.h>

//
// Nunchuck functions
//

// these may need to be adjusted for each nunchuck for calibration
#define ZEROX 510
#define ZEROY 490
#define ZEROZ 460
#define RADIUS 210  // probably pretty universal

#define zeroJoyX 128
#define zeroJoyY 128
static uint8_t ctrlrTypeChuck[6] = {0x00, 0x00, 0xA4, 0x20, 0x00, 0x00};
static uint8_t ctrlrType[6];
static uint8_t nunchuckBuf[6];   // array to store nunchuck data,

static int joyX;
static int joyY;
static int lastJoyX;
static int lastJoyY;
static int joyThreshold = 60;

static int accelX; 
static int accelY;
static int accelZ;

static boolean cButton;
static boolean zButton;
static boolean lastC;
static boolean lastZ;

// Uses port C (analog in) pins as power & ground for Nunchuck
static void nunchuckSetPowerpins()
{
#define pwrpin PORTC3
#define gndpin PORTC2
    DDRC |= _BV(pwrpin) | _BV(gndpin);
    PORTC &=~ _BV(gndpin);
    PORTC |=  _BV(pwrpin);
    delay(100);  // wait for things to stabilize        
}

// initialize the I2C system, join the I2C bus,
// and tell the nunchuck we're talking to it
void nunchuckInit()
{ 
  byte cnt;

Wire.begin();
            
// init controller 
delay(1);
Wire.beginTransmission(0x52);      // device address
Wire.write(0xF0);                    // 1st initialization register
Wire.write(0x55);                    // 1st initialization value
Wire.endTransmission();
delay(1);
Wire.beginTransmission(0x52);
Wire.write(0xFB);                    // 2nd initialization register
Wire.write(0x00);                    // 2nd initialization value
Wire.endTransmission();
delay(1);
            
// read the extension type from the register block        
Wire.beginTransmission(0x52);
Wire.write(0xFA);                    // extension type register
Wire.endTransmission();
Wire.beginTransmission(0x52);
Wire.requestFrom(0x52, 6);               // request data from controller
for (cnt = 0; cnt < 6; cnt++) {
    if (Wire.available()) {
        ctrlrType[cnt] = Wire.read(); // Should be 0x0000 A420 0101 for Classic Controller, 0x0000 A420 0000 for nunchuck 
		//Serial.println(ctrlrType[cnt], HEX);
  }
}
Wire.endTransmission();
delay(1);
            
// send the crypto key (zeros), in 3 blocks of 6, 6 & 4.
Wire.beginTransmission(0x52);
Wire.write(0xF0);                    // crypto key command register
Wire.write(0xAA);                    // sends crypto enable notice
Wire.endTransmission();
delay(1);
Wire.beginTransmission(0x52);
Wire.write(0x40);                    // crypto key data address
for (cnt = 0; cnt < 6; cnt++) {
    Wire.write(0x00);                    // sends 1st key block (zeros)
}
Wire.endTransmission();
Wire.beginTransmission(0x52);
Wire.write(0x40);                    // sends memory address
for (cnt = 6; cnt < 12; cnt++) {
    Wire.write(0x00);                    // sends 2nd key block (zeros)
}
Wire.endTransmission();
Wire.beginTransmission(0x52);
Wire.write(0x40);                    // sends memory address
for (cnt = 12; cnt < 16; cnt++) {
    Wire.write(0x00);                    // sends 3rd key block (zeros)
}
Wire.endTransmission();
delay(1);
// end device init
}

// Send a request for data to the nunchuck
// was "send_zero()"
void nunchuckSendRequest()
{
  Wire.beginTransmission(0x52);	// transmit to device 0x52
  Wire.write(0x00);		// sends one byte
  Wire.endTransmission();	// stop transmitting
}

// Receive data back from the nunchuck, 
int nunchuckUpdate()
{
    int cnt=0;
    Wire.requestFrom (0x52, 6);	// request data from nunchuck
    while (Wire.available ()) {
      // receive byte as an integer
      nunchuckBuf[cnt] = Wire.read(); //nunchuckDecodeByte(Wire.read());
      cnt++;
    }
    nunchuckSendRequest();  // send request for next data payload
    // If we received the 6 bytes, then go print them
    if (cnt >= 5) {
		lastZ = zButton;
		lastC = cButton;
		lastJoyX = joyX;
		lastJoyY = joyY;
	 nunchuckDecodeBuffer();
     return 1;   // success
    }
    return 0; //failure
}

// Print the input data we have received
// accel data is 10 bits long
// so we read 8 bits, then we have to add
// on the last 2 bits.  That is why I
// multiply them by 2 * 2
void nunchuckDecodeBuffer(){ 
	
	joyX = nunchuckBuf[0];
	joyY = nunchuckBuf[1];
	accelX = nunchuckBuf[2]; // * 2 * 2; 
	accelY = nunchuckBuf[3]; // * 2 * 2;
	accelZ = nunchuckBuf[4]; // * 2 * 2;

	// byte nunchuck_buf[5] contains bits for z and c buttons
	// it also contains the least significant bits for the accelerometer data
	// so we have to check each bit of byte outbuf[5]

	zButton = !( nunchuckBuf[5] & B00000001);
	cButton = !((nunchuckBuf[5] & B00000010) >> 1);
	
	if ((nunchuckBuf[5] >> 2) & 1) accelX += 2;
	if ((nunchuckBuf[5] >> 3) & 1) accelX += 1;

	if ((nunchuckBuf[5] >> 4) & 1) accelY += 2;
	if ((nunchuckBuf[5] >> 5) & 1) accelY += 1;

	if ((nunchuckBuf[5] >> 6) & 1) accelZ += 2;
	if ((nunchuckBuf[5] >> 7) & 1) accelZ += 1;
}

// Encode data to format that most wiimote drivers except
// only needed if you use one of the regular wiimote drivers
char nunchuckDecodeByte (char x)
{
  x = (x ^ 0x17) + 0x17;
  return x;
}



// munkey functions to use nunchuck 
// in main code

boolean isNunchuckInitialized(){
	int iAmZeroIfOk = 0;
	for(int cnt = 0; cnt < 6; cnt++){
		if(ctrlrType[cnt] != ctrlrTypeChuck[cnt]){
			// if they not equal we simply count one up
			iAmZeroIfOk++;
		} 
	}
	
	// if iAmZeroIfOk is still Zero then all six numbers are equal and a nunchuck is initialized
	if(iAmZeroIfOk == 0){
		return true;
	}else{
		return false;
	}
}

int readJoyX() {
	return (int)joyX - zeroJoyX;
}
int readJoyY() {
	return (int)joyY - zeroJoyY;
}
float readAccelX() {
	return accelX - ZEROX;
}
float readAccelY() {
	return accelY - ZEROY;
}
float readAccelZ() {
	return accelZ - ZEROZ;
}
boolean JoyLeft(){
	return (readJoyX() < -joyThreshold);
}
boolean JoyRight(){
	return (readJoyX() > joyThreshold);
}
boolean JoyUp(){
	return (readJoyY() > joyThreshold);
}
boolean JoyDown(){
	return (readJoyY() < -joyThreshold);
}

// returns roll degrees
int readRoll() {
	return (int)(atan2(readAccelX(),readAccelZ())/ M_PI * 180.0);
}

// returns pitch in degrees
int readPitch() {
	return (int) (acos(readAccelY()/RADIUS)/ M_PI * 180.0);  // optionally swap 'RADIUS' for 'R()'
}


boolean cButtonPressed(){
	return cButton;
}
boolean zButtonPressed(){
	return zButton;
}